@@ -60,6 +60,12 @@ LOST = 2
 DASH = '-'
 NL = '\n'
 
+try:
+    True, False
+except NameError:
+    True = 1
+    False = 0
+
 
 
 class ListAdmin:
@@ -83,6 +89,8 @@ class ListAdmin:
             except IOError, e:
                 if e.errno <> errno.ENOENT: raise
                 self.__db = {}
+                # put version number in new database
+                self.__db['version'] = IGN, mm_cfg.REQUESTS_FILE_SCHEMA_VERSION
 
     def __closedb(self):
         if self.__db is not None:
@@ -122,7 +130,7 @@ class ListAdmin:
 
     def NumRequestsPending(self):
         self.__opendb()
-        # Subtrace one for the version pseudo-entry
+        # Subtract one for the version pseudo-entry
         return len(self.__db) - 1
 
     def __getmsgids(self, rtype):
