@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2004 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2005 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ from types import *
 
 import email.Iterators
 from email.Utils import getaddresses, formataddr, parseaddr
+from email.Header import Header
 
 from Mailman import mm_cfg
 from Mailman import Utils
@@ -194,8 +195,11 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
     def GetOwnerEmail(self):
         return self.getListAddress('owner')
 
-    def GetRequestEmail(self):
-        return self.getListAddress('request')
+    def GetRequestEmail(self, cookie=''):
+        if mm_cfg.VERP_CONFIRMATIONS and cookie:
+            return self.GetConfirmEmail(cookie)
+        else:
+            return self.getListAddress('request')
 
     def GetConfirmEmail(self, cookie):
         return mm_cfg.VERP_CONFIRM_FORMAT % {
@@ -203,6 +207,26 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             'cookie': cookie,
             } + '@' + self.host_name
 
+    def GetConfirmJoinSubject(self, listname, cookie):
+        if mm_cfg.VERP_CONFIRMATIONS and cookie:
+            cset = Utils.GetCharSet(self.preferred_language)
+            subj = Header(
+     _('Your confirmation is required to join the %(listname)s mailing list'),
+                          cset, header_name='subject')
+            return subj
+        else:
+            return 'confirm ' + cookie
+
+    def GetConfirmLeaveSubject(self, listname, cookie):
+        if mm_cfg.VERP_CONFIRMATIONS and cookie:
+            cset = Utils.GetCharSet(self.preferred_language)
+            subj = Header(
+     _('Your confirmation is required to leave the %(listname)s mailing list'),
+                          cset, header_name='subject')
+            return subj
+        else:
+            return 'confirm ' + cookie
+
     def GetListEmail(self):
         return self.getListAddress()
 
@@ -335,6 +359,9 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         self.include_list_post_header = 1
         self.filter_mime_types = mm_cfg.DEFAULT_FILTER_MIME_TYPES
         self.pass_mime_types = mm_cfg.DEFAULT_PASS_MIME_TYPES
+        self.filter_filename_extensions = \
+            mm_cfg.DEFAULT_FILTER_FILENAME_EXTENSIONS
+        self.pass_filename_extensions = mm_cfg.DEFAULT_PASS_FILENAME_EXTENSIONS
         self.filter_content = mm_cfg.DEFAULT_FILTER_CONTENT
         self.convert_html_to_plaintext = \
             mm_cfg.DEFAULT_CONVERT_HTML_TO_PLAINTEXT
@@ -358,6 +385,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         self.discard_these_nonmembers = []
         self.forward_auto_discards = mm_cfg.DEFAULT_FORWARD_AUTO_DISCARDS
         self.generic_nonmember_action = mm_cfg.DEFAULT_GENERIC_NONMEMBER_ACTION
+        self.nonmember_rejection_notice = ''
         # Ban lists
         self.ban_list = []
         # BAW: This should really be set in SecurityManager.InitVars()
@@ -382,6 +410,10 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             self.encode_ascii_prefixes = 0
         else:
             self.encode_ascii_prefixes = 2
+        # scrub regular delivery
+        self.scrub_nondigest = mm_cfg.DEFAULT_SCRUB_NONDIGEST
+        # automatic discarding
+        self.max_days_to_hold = mm_cfg.DEFAULT_MAX_DAYS_TO_HOLD
 
 
     #
@@ -731,7 +763,6 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         """
         invitee = userdesc.address
         Utils.ValidateEmail(invitee)
-        requestaddr = self.GetRequestEmail()
         # Hack alert!  Squirrel away a flag that only invitations have, so
         # that we can do something slightly different when an invitation
         # subscription is confirmed.  In those cases, we don't need further
@@ -739,6 +770,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # list name to prevent invitees from cross-subscribing.
         userdesc.invitation = self.internal_name()
         cookie = self.pend_new(Pending.SUBSCRIPTION, userdesc)
+        requestaddr = self.getListAddress('request')
         confirmurl = '%s/%s' % (self.GetScriptURL('confirm', absolute=1),
                                 cookie)
         listname = self.real_name
@@ -752,17 +784,13 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
              'cookie'     : cookie,
              'listowner'  : self.GetOwnerEmail(),
              }, mlist=self)
-        if mm_cfg.VERP_CONFIRMATIONS:
-            subj = _(
-                'You have been invited to join the %(listname)s mailing list')
-            sender = self.GetConfirmEmail(cookie)
-        else:
-            # Do it the old fashioned way
-            subj = 'confirm ' + cookie
-            sender = requestaddr
+        sender = self.GetRequestEmail(cookie)
         msg = Message.UserNotification(
-            invitee, sender, subj,
-            text, lang=self.preferred_language)
+            invitee, sender,
+            text=text, lang=self.preferred_language)
+        subj = self.GetConfirmJoinSubject(listname, cookie)
+        del msg['subject']
+        msg['Subject'] = subj
         msg.send(self)
 
     def AddMember(self, userdesc, remote=None):
@@ -869,18 +897,18 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                  'listaddr'    : self.GetListEmail(),
                  'listname'    : realname,
                  'cookie'      : cookie,
-                 'requestaddr' : self.GetRequestEmail(),
+                 'requestaddr' : self.getListAddress('request'),
                  'remote'      : remote,
                  'listadmin'   : self.GetOwnerEmail(),
                  'confirmurl'  : confirmurl,
                  }, lang=lang, mlist=self)
             msg = Message.UserNotification(
-                recipient, self.GetRequestEmail(),
+                recipient, self.GetRequestEmail(cookie),
                 text=text, lang=lang)
             # BAW: See ChangeMemberAddress() for why we do it this way...
             del msg['subject']
-            msg['Subject'] = 'confirm ' + cookie
-            msg['Reply-To'] = self.GetRequestEmail()
+            msg['Subject'] = self.GetConfirmJoinSubject(realname, cookie)
+            msg['Reply-To'] = self.GetRequestEmail(cookie)
             msg.send(self)
             who = formataddr((name, email))
             syslog('subscribe', '%s: pending %s %s',
@@ -1057,7 +1085,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
              'listaddr'   : self.GetListEmail(),
              'listname'   : realname,
              'cookie'     : cookie,
-             'requestaddr': self.GetRequestEmail(),
+             'requestaddr': self.getListAddress('request'),
              'remote'     : '',
              'listadmin'  : self.GetOwnerEmail(),
              'confirmurl' : confirmurl,
@@ -1070,18 +1098,18 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
         # Subject: in a separate step, although we have to delete the one
         # UserNotification adds.
         msg = Message.UserNotification(
-            newaddr, self.GetRequestEmail(),
+            newaddr, self.GetRequestEmail(cookie),
             text=text, lang=lang)
         del msg['subject']
-        msg['Subject'] = 'confirm ' + cookie
-        msg['Reply-To'] = self.GetRequestEmail()
+        msg['Subject'] = self.GetConfirmJoinSubject(realname, cookie)
+        msg['Reply-To'] = self.GetRequestEmail(cookie)
         msg.send(self)
 
     def ApprovedChangeMemberAddress(self, oldaddr, newaddr, globally):
         # It's possible they were a member of this list, but choose to change
         # their membership globally.  In that case, we simply remove the old
         # address.
-        if self.isMember(newaddr):
+        if self.getMemberCPAddress(oldaddr) == newaddr:
             self.removeMember(oldaddr)
         else:
             self.changeMemberAddress(oldaddr, newaddr)
@@ -1101,7 +1129,7 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
             mlist.Lock()
             try:
                 # Same logic as above, re newaddr is already a member
-                if mlist.isMember(newaddr):
+                if mlist.getMemberCPAddress(oldaddr) == newaddr:
                     mlist.removeMember(oldaddr)
                 else:
                     mlist.changeMemberAddress(oldaddr, newaddr)
@@ -1256,18 +1284,18 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
              'listaddr'    : self.GetListEmail(),
              'listname'    : realname,
              'cookie'      : cookie,
-             'requestaddr' : self.GetRequestEmail(),
+             'requestaddr' : self.getListAddress('request'),
              'remote'      : remote,
              'listadmin'   : self.GetOwnerEmail(),
              'confirmurl'  : confirmurl,
              }, lang=lang, mlist=self)
         msg = Message.UserNotification(
-            addr, self.GetRequestEmail(),
+            addr, self.GetRequestEmail(cookie),
             text=text, lang=lang)
             # BAW: See ChangeMemberAddress() for why we do it this way...
         del msg['subject']
-        msg['Subject'] = 'confirm ' + cookie
-        msg['Reply-To'] = self.GetRequestEmail()
+        msg['Subject'] = self.GetConfirmLeaveSubject(realname, cookie)
+        msg['Reply-To'] = self.GetRequestEmail(cookie)
         msg.send(self)
 
 
@@ -1276,20 +1304,19 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
     #
     def HasExplicitDest(self, msg):
         """True if list name or any acceptable_alias is included among the
-        to or cc addrs."""
-        # BAW: fall back to Utils.ParseAddr if the first test fails.
-        # this is the list's full address
+        addresses in the recipient headers.
+        """
+        # This is the list's full address.
         listfullname = '%s@%s' % (self.internal_name(), self.host_name)
         recips = []
-        # check all recipient addresses against the list's explicit addresses,
+        # Check all recipient addresses against the list's explicit addresses,
         # specifically To: Cc: and Resent-to:
         to = []
         for header in ('to', 'cc', 'resent-to', 'resent-cc'):
             to.extend(getaddresses(msg.get_all(header, [])))
         for fullname, addr in to:
-            # It's possible that if the header doesn't have a valid
-            # (i.e. RFC822) value, we'll get None for the address.  So skip
-            # it.
+            # It's possible that if the header doesn't have a valid RFC 2822
+            # value, we'll get None for the address.  So skip it.
             if addr is None:
                 continue
             addr = addr.lower()
@@ -1298,40 +1325,39 @@ class MailList(HTMLFormatter, Deliverer, ListAdmin,
                     localpart == self.internal_name() or
                     # exact match against the complete list address
                     addr == listfullname):
-                return 1
+                return True
             recips.append((addr, localpart))
-        #
-        # helper function used to match a pattern against an address.  Do it
+        # Helper function used to match a pattern against an address.
         def domatch(pattern, addr):
             try:
-                if re.match(pattern, addr):
-                    return 1
+                if re.match(pattern, addr, re.IGNORECASE):
+                    return True
             except re.error:
                 # The pattern is a malformed regexp -- try matching safely,
                 # with all non-alphanumerics backslashed:
-                if re.match(re.escape(pattern), addr):
-                    return 1
-        #
+                if re.match(re.escape(pattern), addr, re.IGNORECASE):
+                    return True
+            return False
         # Here's the current algorithm for matching acceptable_aliases:
         #
         # 1. If the pattern does not have an `@' in it, we first try matching
         #    it against just the localpart.  This was the behavior prior to
-        #    2.0beta3, and is kept for backwards compatibility.
-        #    (deprecated).
+        #    2.0beta3, and is kept for backwards compatibility.  (deprecated).
         #
         # 2. If that match fails, or the pattern does have an `@' in it, we
         #    try matching against the entire recip address.
+        aliases = self.acceptable_aliases.splitlines()
         for addr, localpart in recips:
-            for alias in self.acceptable_aliases.split('\n'):
+            for alias in aliases:
                 stripped = alias.strip()
                 if not stripped:
-                    # ignore blank or empty lines
+                    # Ignore blank or empty lines
                     continue
                 if '@' not in stripped and domatch(stripped, localpart):
-                    return 1
+                    return True
                 if domatch(stripped, addr):
-                    return 1
-        return 0
+                    return True
+        return False
 
     def parse_matching_header_opt(self):
         """Return a list of triples [(field name, regex, line), ...]."""
@@ -1378,13 +1404,17 @@ bad regexp in bounce_matching_header line: %s
                     return line
         return 0
 
-    def autorespondToSender(self, sender):
+    def autorespondToSender(self, sender, lang=None):
         """Return true if Mailman should auto-respond to this sender.
 
         This is only consulted for messages sent to the -request address, or
         for posting hold notifications, and serves only as a safety value for
         mail loops with email 'bots.
         """
+        # language setting
+        if lang == None:
+            lang = self.preferred_language
+        i18n.set_language(lang)
         # No limit
         if mm_cfg.MAX_AUTORESPONSES_PER_DAY == 0:
             return 1
@@ -1412,11 +1442,12 @@ bad regexp in bounce_matching_header line: %s
                  'listname': '%s@%s' % (self.real_name, self.host_name),
                  'num' : count,
                  'owneremail': self.GetOwnerEmail(),
-                 })
+                 },
+                lang=lang)
             msg = Message.UserNotification(
                 sender, self.GetOwnerEmail(),
                 _('Last autoresponse notification for today'),
-                text)
+                text, lang=lang)
             msg.send(self)
             return 0
         self.hold_and_cmd_autoresponses[sender] = (today, count+1)
