@@ -1,4 +1,4 @@
-# Copyright (C) 2000-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 2000-2005 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -102,11 +102,14 @@ def prepare_message(mlist, msg, msgdata):
         del msg['approved']
         msg['Approved'] = mlist.GetListEmail()
     # Should we restore the original, non-prefixed subject for gatewayed
-    # messages?
-    origsubj = msgdata.get('origsubj')
-    if not mlist.news_prefix_subject_too and origsubj is not None:
+    # messages? TK: We use stripped_subject (prefix stripped) which was
+    # crafted in CookHeaders.py to ensure prefix was stripped from the subject
+    # came from mailing list user.
+    stripped_subject = msgdata.get('stripped_subject') \
+                       or msgdata.get('origsubj')
+    if not mlist.news_prefix_subject_too and stripped_subject is not None:
         del msg['subject']
-        msg['subject'] = origsubj
+        msg['subject'] = stripped_subject
     # Add the appropriate Newsgroups: header
     ngheader = msg['newsgroups']
     if ngheader is not None:
