@@ -6,7 +6,6 @@ http://linux.gda.pl/pub/Mailman-pl/
 Lista dyskusyjna polskich tumaczy:
 http://linux.gda.pl/mailman/listinfo/mailman-pl
 
-
 Do tej pory udzia w tumaczeniu wzieli (w kolejnoci chronologicznej
 przyczania si do projektu):
 - Marcin Sochacki <wanted%linux.gda.pl>
@@ -15,16 +14,20 @@ przyczania si do projektu):
 - Wojciech Czarnowski <wojtekcz%rodos.com.pl>
 - Bartosz Sawicki <sawickib%iem.pw.edu.pl>
 - Robert wicki <rswieck1%elka.pw.edu.pl>
-
+- Pawe Goaszewski <blues@ds.pg.gda.pl>
 
 W chwili obecnej przetumaczone jest 99% interfejsu subskrybenta i okoo 
-80% interfejsu administracyjnego. Najbardziej aktualnej wersji szukajcie w
+95% interfejsu administracyjnego. Najbardziej aktualnej wersji szukajcie w
 oficjalnych CVS Mailmana. 
 
 Do zrobienia:
-- przetumaczenie README.*
-- szlifowanie, polerowania :)
+- szlifowanie, polerowanie :)
+- skoczenie tumaczenia mailman.po (zostao jeszcze 230 msgid)
+- przetumaczenie README.* dla rnych MTA
 
-2003-09-25, sawickib
+Od jakiego roku nad tumaczeniem pracuj wlaciwie tylko sam, ale bardzo
+chtnie przyjm wszystkie formy pomocy.
+
+2004-12-30, Bartosz Sawicki <sawickib@iem.pw.edu.pl>
 
 
