@@ -1,4 +1,4 @@
-# Copyright (C) 2002 Python Software Foundation
+# Copyright (C) 2002-2004 Python Software Foundation
 
 """A simple test runner, which sets up sys.path properly.
 
@@ -23,11 +23,18 @@ try:
     from email.test import test_email_torture
 except TestSkipped:
     test_email_torture = None
+
+# See if we have the Japanese codecs package installed
 try:
-    # This will raise a TestSkipped if the Japanese codecs aren't installed
+    # Python 2.4
+    unicode('foo', 'euc-jp')
+except LookupError:
+    try:
+        unicode('foo', 'japanese.iso-2022-jp')
+    except LookupError:
+        test_email_codecs = None
+else:
     from email.test import test_email_codecs
-except TestSkipped:
-    test_email_codecs = None
 
 
 
